--
with Ada.Strings.Fixed;
with Ada.Float_Text_IO;
with Ada.Long_Float_Text_IO;

with Ada.Numerics;

with Logs_Buffer;

package body Io_Support is
   use Ada;
   use Strings;
   use Strings.Fixed;
   use Float_Text_IO;

   Sep : constant String := " | ";

   -- -- --

   function Str (V : Boolean) return String is
   begin
      return Trim (Boolean'Image (V), Left) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Str;

   -- -- --

   function Str (V : Integer) return String is
   begin
      return Trim (Integer'Image (V), Left) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Str;

   -- -- --

   function Dot_To_Comma (S : String) return String;
   function Dot_To_Comma (S : String) return String is
   begin

      return Fixed.Overwrite (S, Fixed.Index (S, "."), ",");

   exception
      when others =>
         return S;
   end Dot_To_Comma;

   -- -- --

   function Str (V : Float) return String is
   begin
      return Dot_To_Comma (Trim (Float'Image (V), Left)) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Str;

   -- -- --

   function Str (V : Long_Float) return String is
   begin
      return Dot_To_Comma (Trim (Long_Float'Image (V), Left)) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Str;

   -- -- --

   function Put
     (V    : Float;
      Aft  : Field := Float'Digits - 1;
      Exp  : Field := 3)
      return String is
      To_Str : String (1 .. 12);
   begin
      Float_Text_IO.Put (To_Str, V, Aft, Exp);

      return Dot_To_Comma (Trim (To_Str, Left)) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Put;

   -- -- --

   function Put
     (V    : Long_Float;
      Aft  : Field := Long_Float'Digits - 1;
      Exp  : Field := 3)
      return String is
      To_Str : String (1 .. 22);
   begin
      Long_Float_Text_IO.Put (To_Str, V, Aft, Exp);

      return Dot_To_Comma (Trim (To_Str, Left)) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Put;

   -- -- --

   function Generic_Put_Discrete (V : Discrete_Type) return String is
   begin

      declare
         Img : String renames Discrete_Type'Image (V);
      begin
         return Img & Sep;
      end; --  declare

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Generic_Put_Discrete;

   -- -- --

   Convf : constant := 3200.0 / Ada.Numerics.Pi;

   function Mils (V : Float) return String is
   begin
      return Str (Integer (V * Convf)) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Mils;

   function Mils (V : Long_Float) return String is
   begin
      return Str (Integer (V * Convf)) & Sep;

   exception
      when others =>
         return "<ERROR>" & Sep;
   end Mils;

   -- -- --
   -- -- --

   function Generic_Put_Discrete_Array
     (A     : Array_Type;
      First : Index_Type := Index_Type'First;
      Last  : Index_Type := Index_Type'Last)
      return  String is

      use Logs_Buffer.Bounded_Ms;

      Bds : Bounded_String := Null_Bounded_String;
   begin

      for Index in First .. Last loop

         declare
            Img : String renames Discrete_Type'Image (A (Index));
         begin
            Bds := Bds & Img & Sep;
         end;

      end loop;

      return To_String (Bds);

   exception
      when E : others =>
         return "<ERROR>" & Sep;
   end Generic_Put_Discrete_Array;

   -- -- --

   function Generic_Put_Discrete_Constrained_Array (A : Array_Type) return String is
      use Logs_Buffer.Bounded_Ms;

      Bds : Bounded_String := Null_Bounded_String;
   begin
      for Index in A'Range loop

         declare
            Img : String renames Discrete_Type'Image (A (Index));
         begin
            Bds := Bds & Img & Sep;
         end;
      end loop;

      return To_String (Bds);

   exception
      when E : others =>
         return "<ERROR>" & Sep;
   end Generic_Put_Discrete_Constrained_Array;

   -- -- --

   function Generic_Put_Floating_Array
     (A     : in Array_Type;
      First : Index_Type := Index_Type'First;
      Last  : Index_Type := Index_Type'Last)
      return  String is
      use Logs_Buffer.Bounded_Ms;

      Bds : Bounded_String := Null_Bounded_String;
   begin

      for Index in First .. Last loop

         declare
            Img : String renames Floating_Type'Image (A (Index));
         begin
            Bds := Bds & (Dot_To_Comma (Trim (Img, Left))) & Sep;
         end;

      end loop;

      return To_String (Bds);

   exception
      when E : others =>
         return "<ERROR>" & Sep;

   end Generic_Put_Floating_Array;

   -- -- --

   function Generic_Put_Floating_Constrained_Array (A : in Array_Type) return String is
      use Logs_Buffer.Bounded_Ms;

      Bds : Bounded_String := Null_Bounded_String;
   begin

      for Index in A'Range loop

         declare
            Img : String renames Floating_Type'Image (A (Index));
         begin
            Bds := Bds & (Dot_To_Comma (Trim (Img, Left))) & Sep;
         end;

      end loop;

      return To_String (Bds);

   exception
      when E : others =>
         return "<ERROR>" & Sep;

   end Generic_Put_Floating_Constrained_Array;

   -- -- --

end Io_Support;
