--

with Ada.Text_IO;
with Ada.Exceptions;

package body Logs_To_Ram.Re is
   use Ada;

   -- -- --

   function Compile (P : String; Cs : Boolean := False) return Regexp is
      Re : Regexp;
   begin
      Re := Reg_Exp.Compile (Pattern => P, Case_Sensitive => Cs);

      return Re;

   exception
      when others =>
         Problems_With_Data_Logging := True;

         raise;
   end Compile;

   -- -- --
   -- -- --

   procedure Append_To_Log_With_Re
     (B  : in Buff_Enum_Type;
      S1 : in String;
      S2 : in String;
      Re : in Regexp;
      S3 : String := "") is
   begin

      if not Do_The_Logging then
         return;
      elsif Problems_With_Data_Logging then
         return;
      end if;

      if (S1 = "") or (S2 = "") then
         return;
      end if;

      if not Reg_Exp.Match (S2, Re) then
         Strings_Store_Array (B) := Null_Bounded_String;

         return;
      end if;

      -- -- --

      Bounded_Ms.Append (Strings_Store_Array (B), S1 & S2 & S3, Strings.Right);

   exception
      when Event : others =>
         Text_IO.Put ("!!!! Logs_To_Ram.Re - Exception with Append_To_Log_With_Re: ");
         Text_IO.Put_Line (Exceptions.Exception_Information (Event));
         Text_IO.Flush;

         Problems_With_Data_Logging := True;
   end Append_To_Log_With_Re;

   -- -- --

   procedure Append_To_Log_With_Re
     (Pos : in Pos_Nb_Type;
      S1  : in String;
      --
      S2  : in String;
      Re  : in Regexp;
      --
      S3  : String := "") is

      B : Buff_Enum_Type;
   begin

      B := Buff_Enum_Type'Val (Pos);

      Append_To_Log_With_Re (B, S1, S2, Re, S3);

   end Append_To_Log_With_Re;

   -- -- --
   -- -- --

   Re_Store_Array : array (Buff_Enum_Type) of Regexp;

   -- -- --

   procedure Compile (B : in Buff_Enum_Type; P : in String; Cs : in Boolean := False) is
   begin
      Re_Store_Array (B) := Reg_Exp.Compile (Pattern => P, Case_Sensitive => Cs);

   exception
      when others =>
         null;
   end Compile;

   -- -- --

   procedure Compile (Pos : in Pos_Nb_Type; P : in String; Cs : in Boolean := False) is
      B : Buff_Enum_Type;
   begin

      B := Buff_Enum_Type'Val (Pos);

      Compile (B, P, Cs);

   end Compile;

   -- -- --

   procedure Append_To_Log_With_Re
     (B  : in Buff_Enum_Type;
      S1 : in String;
      S2 : in String;
      S3 : in String := "") is
   begin

      Append_To_Log_With_Re (B, S1, S2, Re_Store_Array (B), S3);

   end Append_To_Log_With_Re;

   -- -- --

   procedure Append_To_Log_With_Re
     (Pos : in Pos_Nb_Type;
      S1  : in String;
      S2  : in String;
      S3  : in String := "") is

      B : Buff_Enum_Type;
   begin

      B := Buff_Enum_Type'Val (Pos);

      Append_To_Log_With_Re (B, S1, S2, S3);

   end Append_To_Log_With_Re;

   -- -- --

end Logs_To_Ram.Re;
