--
with Ada.Text_IO;
with Ada.Strings;

with Io_Support;
with Logs_Buffer;

package Logs_To_Ram is
   use Ada.Text_IO;

   subtype Buff_Enum_Type is Logs_Buffer.Buff_Enum_Type;

   type Pos_Nb_Type is new Natural range
     Buff_Enum_Type'Pos (Buff_Enum_Type'First) .. Buff_Enum_Type'Pos (Buff_Enum_Type'Last);

   -- -- --

   function Sep (Nb : Positive := 1) return String;
   --
   --  Separator for string parts

   -- -- --
   -- -- --

   --  The following routines do add a separator " | "
   --  at the end of the returned string.

   function Str (V : Boolean) return String;
   function Str (V : Integer) return String;
   function Str (V : Float) return String;
   function Str (V : Long_Float) return String;

   -- -- --

   function Mils (V : Float) return String;
   function Mils (V : Long_Float) return String;

   -- -- --

   function Put
     (V    : in Float;
      Aft  : in Field := Float'Digits - 1;
      Exp  : in Field := 3)
      return String;

   function Put
     (V    : in Long_Float;
      Aft  : in Field := Long_Float'Digits - 1;
      Exp  : in Field := 3)
      return String;

   -- -- --

   generic function Generic_Put_Discrete renames Io_Support.Generic_Put_Discrete;

   generic function Generic_Put_Discrete_Array renames Io_Support.Generic_Put_Discrete_Array;
   generic function Generic_Put_Discrete_Constrained_Array renames Io_Support.Generic_Put_Discrete_Constrained_Array;

   generic function Generic_Put_Floating_Array renames Io_Support.Generic_Put_Floating_Array;
   generic function Generic_Put_Floating_Constrained_Array renames Io_Support.Generic_Put_Floating_Constrained_Array;

   -- -- --
   -- -- --

   --  Do use a separator Sep to partition the string in parts for use with Excel

   procedure Do_Log_To_Ram (Pos : in Pos_Nb_Type; S : in String; F : in String := "");
   --
   --  The log item S is logged to RAM only when S contains the filter
   --  string F (for F /= "")
   --
   --  Note: To compare with F (only!) the string S is converted to lower case

   -- -- --
   -- -- --

   procedure Append_To_Log_Item (Pos : in Pos_Nb_Type; S : in String);
   --
   --  Use the procedure one or more times to build up the whole string
   --  To be logged at the end with Do_Log_All_To_Ram

   --  Precondition: One or more usages of Append_To_Log
   --
   procedure Do_Log_Item_To_Ram (Pos : in Pos_Nb_Type; F : in String := "");
   --
   --  The log item built with Append_To_Log_Item is logged to RAM only when S
   --  contains the filter string F (for F /= "")
   --
   --  Note: To compare with F (only!) the string S is converted to lower case

   -- -- --
   -- -- --

   procedure Save_Logs_To_File;

   -- -- --
   -- -- --

private

   -- -- --

   Do_The_Logging : Boolean := False;
   pragma Atomic (Do_The_Logging);

   Problems_With_Data_Logging : Boolean := False;
   pragma Atomic (Problems_With_Data_Logging);

   -- -- --

   package Bounded_Ms renames Logs_Buffer.Bounded_Ms;
   use Bounded_Ms;

   Strings_Store_Array : array (Buff_Enum_Type) of Bounded_String := (others => Null_Bounded_String);

   procedure Do_Log_To_Ram (B : in Buff_Enum_Type; S : in String; F : in String := "");
   procedure Append_To_Log_Item (B : in Buff_Enum_Type; S : in String);
   procedure Do_Log_Item_To_Ram (B : in Buff_Enum_Type; F : in String := "");

end Logs_To_Ram;
