--

with Ada.Calendar;

package Ts.Lada is
   use Ada;

   subtype Year_Type is Calendar.Year_Number;
   subtype Month_Type is Calendar.Month_Number;
   subtype Day_Type is Calendar.Day_Number;

   type Hour_Type is range 0 .. 23;
   type Minute_Type is range 0 .. 59;
   type Second_Type is range 0 .. 59;
   type Millisecond_Type is range 0 .. 999;

   type Date_And_Time_Record_Type is record
      Year        : Year_Type;
      Month       : Month_Type;
      Day         : Day_Type;
      Hour        : Hour_Type;
      Minute      : Minute_Type;
      Second      : Second_Type;
      Millisecond : Millisecond_Type;
   end record;

   -- -- --

   function Date_And_Time_Clock return Date_And_Time_Record_Type;
   --
   --  Returns the Calendar.Clock in Date_And_Time_Record_Type format

   function Image (Date : Date_And_Time_Record_Type) return String;
   --
   --  String format is : "YYYY-MM-DD HH:MM:SS.mmm"

   function Stamp_Time return String;
   --
   --  String format is : "YYYY-MM-DD HH:MM:SS.mmm"

   function Short_Image (Date : Date_And_Time_Record_Type) return String;
   --
   --  String format is : "YYYYMMDD_HHMMSS"

   function Stamp_Short_Time return String;
   --
   --  String format is : "YYYYMMDD_HHMMSS"

   -- -- --
   -- -- --

   subtype Time_Type is Ada.Calendar.Time;

   -- -- --

   function Clock return Time_Type renames Ada.Calendar.Clock;

   function Image (T : Time_Type) return String;
   --
   --  String format is : "YYYY-MM-DD HH:MM:SS.mmm"

   function Image_In_Seconds (T : Time_Type) return String;

   function Stamp_Time_In_Seconds return String;
   --
   -- String format is : 12345.123456

   -- -- --

end Ts.Lada;
