--

with Ada.Text_IO;
with Ada.Integer_Text_IO;

package body Ts.Lada is
   use Ada;

   type Duration_Type is digits 15;

   type Day_Nb_Type is new Integer range
     -Integer (Year_Type'Last - Year_Type'First + 1) * 366 .. Integer (Year_Type'Last - Year_Type'First + 1) * 366;
   --
   --  365.25 rounded by excess

   type Working_Type is digits 15;

   Ct_Nb_Of_Seconds_In_A_Day    : constant := 86_400.0;
   Ct_Nb_Of_Seconds_In_A_Hour   : constant := 3_600.0;
   Ct_Nb_Of_Seconds_In_A_Minute : constant := 60.0;
   Ct_Nb_Of_Seconds_In_A_Ms     : constant := 0.001;

   Ct_Average_Days_In_Year : constant := 365.250001;

   type Extended_Month_Type is range 1 .. 13;

   type Day_Before_Month_Array_Type is array (Extended_Month_Type) of Day_Nb_Type;

   Ct_Normal_Days_Before_Month : constant Day_Before_Month_Array_Type :=
     (1  => 0,
      2  => 31,
      3  => 31 + 28,
      4  => 31 + 28 + 31,
      5  => 31 + 28 + 31 + 30,
      6  => 31 + 28 + 31 + 30 + 31,
      7  => 31 + 28 + 31 + 30 + 31 + 30,
      8  => 31 + 28 + 31 + 30 + 31 + 30 + 31,
      9  => 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31,
      10 => 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30,
      11 => 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31,
      12 => 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30,
      13 => 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30 + 31);
   --
   --  Days before

   Ct_Bissextile_Days_Before_Month : constant Day_Before_Month_Array_Type :=
     (1  => 0,
      2  => 31,
      3  => 31 + 29,
      4  => 31 + 29 + 31,
      5  => 31 + 29 + 31 + 30,
      6  => 31 + 29 + 31 + 30 + 31,
      7  => 31 + 29 + 31 + 30 + 31 + 30,
      8  => 31 + 29 + 31 + 30 + 31 + 30 + 31,
      9  => 31 + 29 + 31 + 30 + 31 + 30 + 31 + 31,
      10 => 31 + 29 + 31 + 30 + 31 + 30 + 31 + 31 + 30,
      11 => 31 + 29 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31,
      12 => 31 + 29 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30,
      13 => 31 + 29 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30 + 31);
   --
   --  Days before

   --  The generic function returns the Integer Part of a real, the greatest integer
   --  smaller than the original parameter
   --
   generic
      type Real_Type is digits <>;
   function Ent_Template (Real : in Real_Type) return Real_Type;

   function Ent_Template (Real : in Real_Type) return Real_Type is
      Ent_Real : Real_Type := Real_Type (Integer (Real));
   begin
      if Ent_Real <= Real then
         return Ent_Real;
      else
         return (Ent_Real - 1.0);
      end if;
   end Ent_Template;

   -- -- --

   function Ent is new Ent_Template (Working_Type);
   function Ent is new Ent_Template (Duration_Type);

   -- -- --

   function Day_Nb_In_Date (Date : in Date_And_Time_Record_Type) return Day_Nb_Type is
      Temp_Day_Nb : Day_Nb_Type;
   begin
      Temp_Day_Nb := Day_Nb_Type (Ent (Working_Type (Date.Year - Year_Type'First) * Ct_Average_Days_In_Year));
      if (Date.Year mod 4 = 0) then  --  if YEAR is bissextile
         Temp_Day_Nb := Temp_Day_Nb + Ct_Bissextile_Days_Before_Month (Extended_Month_Type (Date.Month));
      else
         Temp_Day_Nb := Temp_Day_Nb + Ct_Normal_Days_Before_Month (Extended_Month_Type (Date.Month));
      end if;
      return Temp_Day_Nb + Day_Nb_Type (Date.Day);
   end Day_Nb_In_Date;

   -- -- --

   function Remaining_Seconds_In_Date (Date : in Date_And_Time_Record_Type) return Duration_Type;
   pragma Inline (Remaining_Seconds_In_Date);

   function Remaining_Seconds_In_Date (Date : in Date_And_Time_Record_Type) return Duration_Type is
   begin
      return Duration_Type (Date.Hour) * Ct_Nb_Of_Seconds_In_A_Hour +
             Duration_Type (Date.Minute) * Ct_Nb_Of_Seconds_In_A_Minute +
             Duration_Type (Date.Second) +
             Duration_Type (Date.Millisecond) * Ct_Nb_Of_Seconds_In_A_Ms;
   end Remaining_Seconds_In_Date;

   --  -- -

   function "+" (Left : Date_And_Time_Record_Type; Right : Duration_Type) return Date_And_Time_Record_Type is
      Days_In_Duration              : Day_Nb_Type;
      Remaining_Seconds_In_Duration : Duration_Type;

      Reference_Day_Nb            : Day_Nb_Type;
      Reference_Remaining_Seconds : Duration_Type;

      Final_Day_Nb            : Day_Nb_Type;
      Final_Remaining_Seconds : Duration_Type;
      Final_Date              : Date_And_Time_Record_Type;

   begin

      Days_In_Duration              := Day_Nb_Type (Right / Ct_Nb_Of_Seconds_In_A_Day);
      Remaining_Seconds_In_Duration := Right - Ct_Nb_Of_Seconds_In_A_Day * Duration_Type (Days_In_Duration);

      Reference_Day_Nb            := Day_Nb_In_Date (Left);
      Reference_Remaining_Seconds := Remaining_Seconds_In_Date (Left);

      Final_Remaining_Seconds := Reference_Remaining_Seconds + Remaining_Seconds_In_Duration;

      if Final_Remaining_Seconds >= Ct_Nb_Of_Seconds_In_A_Day then

         Final_Remaining_Seconds := Final_Remaining_Seconds - Ct_Nb_Of_Seconds_In_A_Day;
         Final_Day_Nb            := Reference_Day_Nb + Days_In_Duration + 1;

      elsif Final_Remaining_Seconds < 0.0 then
         Final_Remaining_Seconds := Final_Remaining_Seconds + Ct_Nb_Of_Seconds_In_A_Day;
         Final_Day_Nb            := Reference_Day_Nb + Days_In_Duration - 1;  --  onw day less due

      else
         Final_Day_Nb := Reference_Day_Nb + Days_In_Duration;
      end if;

      Final_Date.Hour         := Hour_Type (Ent (Final_Remaining_Seconds / Ct_Nb_Of_Seconds_In_A_Hour));
      Final_Remaining_Seconds := Final_Remaining_Seconds - Duration_Type (Final_Date.Hour) * Ct_Nb_Of_Seconds_In_A_Hour;

      Final_Date.Minute       := Minute_Type (Ent (Final_Remaining_Seconds / Ct_Nb_Of_Seconds_In_A_Minute));
      Final_Remaining_Seconds := Final_Remaining_Seconds -
                                 Duration_Type (Final_Date.Minute) * Ct_Nb_Of_Seconds_In_A_Minute;

      Final_Date.Second       := Second_Type (Ent (Final_Remaining_Seconds));
      Final_Remaining_Seconds := Final_Remaining_Seconds - Duration_Type (Final_Date.Second);

      if Final_Remaining_Seconds >= 0.999 then
         Final_Date.Millisecond := 999;

      else
         Final_Date.Millisecond := Millisecond_Type (Final_Remaining_Seconds / Ct_Nb_Of_Seconds_In_A_Ms);
      end if;

      Final_Date.Year :=
        Year_Type (Day_Nb_Type (Year_Type'First) +
                   Day_Nb_Type (Ent (Working_Type (Final_Day_Nb) / Ct_Average_Days_In_Year)));

      Final_Day_Nb := Final_Day_Nb -
                      Day_Nb_Type (Ent (Working_Type (Final_Date.Year - Year_Type'First) * Ct_Average_Days_In_Year));

      if (Final_Date.Year mod 4) = 0 then

         for Month in Extended_Month_Type range 1 .. 12 loop

            if Final_Day_Nb in
                 Ct_Bissextile_Days_Before_Month (Month) + 1 .. Ct_Bissextile_Days_Before_Month (Month + 1) then
               Final_Date.Month := Month_Type (Month);
               exit;
            end if;
         end loop;

         Final_Date.Day :=
           Day_Type (Final_Day_Nb - Ct_Bissextile_Days_Before_Month (Extended_Month_Type (Final_Date.Month)));

      else
         for Month in Extended_Month_Type range 1 .. 12 loop

            if Final_Day_Nb in Ct_Normal_Days_Before_Month (Month) + 1 .. Ct_Normal_Days_Before_Month (Month + 1) then

               Final_Date.Month := Month_Type (Month);
               exit;
            end if;

         end loop;

         Final_Date.Day := Day_Type (Final_Day_Nb - Ct_Normal_Days_Before_Month (Extended_Month_Type (Final_Date.Month)));

      end if;

      return Final_Date;
   end "+";

   -- -- --

   package Int_Io renames Ada.Integer_Text_IO;

   -- -- --

   function Int_Image (Num : Integer; Width : Integer := 4) return String is
      S : String (1 .. Width);
   begin
      Int_Io.Put (S, Num);

      for C in S'Range loop

         if S (C) = ' ' then

            S (C) := '0';
         end if;

      end loop;

      return S;
   end Int_Image;

   -- -- --

   function Image (Date : Date_And_Time_Record_Type) return String is
   begin
      return Int_Image (Integer (Date.Year), 4) &
             "-" &
             Int_Image (Integer (Date.Month), 2) &
             "-" &
             Int_Image (Integer (Date.Day), 2) &
             " " &
             Int_Image (Integer (Date.Hour), 2) &
             ":" &
             Int_Image (Integer (Date.Minute), 2) &
             ":" &
             Int_Image (Integer (Date.Second), 2) &
             "." &
             Int_Image (Integer (Date.Millisecond), 3);
   end Image;

   -- -- --

   function Date_And_Time (T : Time_Type) return Date_And_Time_Record_Type is
      Result        : Date_And_Time_Record_Type;
      Today_Seconds : Duration_Type;
   begin
      Calendar.Split
        (Date    => T,
         Year    => Result.Year,
         Month   => Result.Month,
         Day     => Result.Day,
         Seconds => Calendar.Day_Duration (Today_Seconds));

      Result.Hour        := 0;
      Result.Minute      := 0;
      Result.Second      := 0;
      Result.Millisecond := 0;
      Result             := Result + Today_Seconds;

      return Result;

   end Date_And_Time;

   -- -- --

   function Date_And_Time_Clock return Date_And_Time_Record_Type is
   begin
      return Date_And_Time (Calendar.Clock);
   end Date_And_Time_Clock;

   -- -- --

   function Stamp_Time return String is
   begin
      return Image (Date_And_Time_Clock);
   end Stamp_Time;

   -- -- --

   function Short_Image (Date : Date_And_Time_Record_Type) return String is
   begin
      return Int_Image (Integer (Date.Year), 4) &
             Int_Image (Integer (Date.Month), 2) &
             Int_Image (Integer (Date.Day), 2) &
             "_" &
             Int_Image (Integer (Date.Hour), 2) &
             Int_Image (Integer (Date.Minute), 2) &
             Int_Image (Integer (Date.Second), 2);
   end Short_Image;

   -- -- --

   function Stamp_Short_Time return String is
   begin
      return Short_Image (Date_And_Time_Clock);
   end Stamp_Short_Time;

   -- -- --
   -- -- --

   function Image_In_Seconds (T : Time_Type) return String is
      use Ada.Calendar;
   begin
      return Day_Duration'Image (Seconds (T)) (1 .. 13);
   end Image_In_Seconds;

   -- -- --

   function Stamp_Time_In_Seconds return String is
   begin
      return Image_In_Seconds (Clock);
   end Stamp_Time_In_Seconds;

   -- -- --

   function Image (T : Time_Type) return String is
   begin
      return Image (Date_And_Time (T));
   end Image;

   -- -- --

end Ts.Lada;
